/*
 * Copyright (C) 1998-2001 Wolfgang Moser <womo@wmsr.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Commodore CBM 1581 floppy disk copy util for PC's, CENTRAL.H
 *
 * Redirected contact addresses:
 *   Wolfgang Moser <womo@wmsr.de>
 *   Wolfgang Moser <womo@fairlight.to>
 *    http://wmsr.de
 *
 * Current workplace (up to December 2001):
 *   Wolfgang Moser <womo@gm.fh-koeln.de>
 *    http://www2.gm.fh-koeln.de/~womo
 *
 *
 * Basic informations from Dan Fandrich <danf@npsnet.com>.
 *   His readme of the cbmfs-0.3 driver for Linux explained me, what the
 *   difference between a DOS formatted 800 kb disk and a CBM 1581 disk is.
 *   (check: http://vanbc.wimsey.com/~danf/software
 *           http://www.npsnet.com/danf/software)
 *
 *
 * Basic implementations by Ciriaco Garca de Celis <ciri@gui.uva.es>
 *   His util 765Debug, Version 5.0 is great for learning dma based
 *   direct floppy disk controller programming.
 *   (check: ftp://ftp.vse.cz/pub/misc/2m/current/765d50sr.zip)
 *
 * Check out for his floppy disk utils 2M and 2MGUI, the last words
 * in improving floppy disk storage capacity.
 *   ftp://ftp.vse.cz/pub/misc/2m/current
 *
 *
 * For additional informations to FDC programming check:
 *     http://developer.intel.com/design/periphrl/datashts/290468.htm
 *     http://developer.intel.com/design/periphrl/datashts/290474.htm
 *   and get the intel 82078 CHMOS Single-Chip Floppy Disk Controller
 *   PDF document:
 *     ftp://download.intel.com/design/periphrl/datashts/29046803.pdf
 *     ftp://download.intel.co.jp/design/periphrl/datashts/29046803.pdf
 *     ftp://download.intel.com/design/periphrl/datashts/29047403.pdf
 *     ftp://download.intel.co.jp/design/periphrl/datashts/29047403.pdf
 *
 *
 *   National Semiconductor has also some pages about their PC
 *   compatible controllers:
 *     http://www.national.com/pf/DP/DP8473.html
 *     http://www.national.com/pf/PC/PC8477B.html
 *
 *
 * Another good source for floppy disk controller programming information
 * are the linux kernal sources, you could have a look into:
 *     http://src.openresources.com/linux-2.2.1/S/include%20linux%20fdreg.h.html
 *     http://src.openresources.com/linux-2.2.1/S/drivers%20block%20floppy.c.html
 * especially the parameter table explanation
 *     http://src.openresources.com/linux-2.2.1/S/drivers%20block%20floppy.c.html#332
 *
 *
 * Now, that I started talking about Linux: Check out the most massive
 * attack against floppy disks ever made, Alain Knaff's fdutils:
 *     http://alain.knaff.linux.lu/fdutils/index.html
 *     http://fdutils.linux.lu/index.html
 */

#ifndef __Central_1581COPY__
#define __Central_1581COPY__

// version information

#if 1

#define _1581COPY_version_ \
"1581-Copy/FLT, ver. 0.53, 200102111844. "\
"Copyright (C) 1998-2001 Wolfgang Moser\n"

#else

#define _1581COPY_version_ \
"1581-Copy/FLT, V0.53.04, 200102111844. "\
"Copyright (C) 1998-2001 Wolfgang Moser\n"

#endif

/*
 * Most important definitions of 1581-Copy
 */

#define CRITICALBUGWARNING

#define MaxSectors 10					// This controls the size of the internal buffer
												// that is needed for formatting and single sector track
												// copying, it must be greater than or equal to the
												// number of sectors per side (track), that may be used
												// with a desired disk layout

#define TrBufSize 10*1024				// This controls the size of the track buffer, it must
												// be big enough to hold all the sector data of one side
												// currently: 10 sectors of 1024 bytes for FD2000 disks

#define DISABLE_INTS						// disabling INTs, when programming the DMA controller

#define TimeSlice							// give back remaining time slices to the OS
#define TimeEstimation
#define FD2000BAMSupport

//	#define MESSAGES 3					// enable some debugging messages
												// >= 6  - enable all messages
                                    // >= 5  - enable all high level code messages
                                    // >= 3  - enable working stage high level info
                                    // >= 2  - enable sector processing high level info
                                    // >= 1  - enable all main program messages
                                    // >= 0  - main program error reportations

//	#define USE_TSC						// use system timestamp counter for debug measurements
//	#define USE_P_RDTSC					// use pentium timestamp counter instead of system timer

//	#define ForceFnWVerify				// support for the intel 82078 FDC is enabled now,
												// but do the verify until the test phase is over

//	#define ScanEqual						// enable real a comparison instead of simple CRC verify
												// scanEqual doesn't work, so it let the processor
												// compare, but this is much too lame
			// needs an extra interleave after the verify operation
         // the comparison speed would have to be increased

//	#define CreateMaverickDisk	0		// enabling special (wrong) formatting to create
												// Maverick like disks for test purposes
                                    // define the number to store in the side ID field
//	#define CreateMaverickDisk	3		// to test the swap ID check disabling
//	#define CreateMaverickDisk	i		// store the number of the sector into the side ID field
                                    // this is a format, that 1581-Copy definetly cannot handle
//	#define CreateMaverickDisk	side	// the correct CBM1581 format
//	#define CreateMaverickDisk	!side	// this one would be a correct DOS like format but without a
												// correct bootblock, cannot be read with DOS tools normally
#endif

