/*
 * Copyright (C) 1998-2001 Wolfgang Moser <womo@wmsr.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Commodore CBM 1581 floppy disk copy util for PC's, OPERATNS.H
 *
 * Redirected contact addresses:
 *   Wolfgang Moser <womo@wmsr.de>
 *   Wolfgang Moser <womo@fairlight.to>
 *    http://wmsr.de
 *
 * Current workplace (up to December 2001):
 *   Wolfgang Moser <womo@gm.fh-koeln.de>
 *    http://www2.gm.fh-koeln.de/~womo
 *
 *
 * Basic informations from Dan Fandrich <danf@npsnet.com>.
 *   His readme of the cbmfs-0.3 driver for Linux explained me, what the
 *   difference between a DOS formatted 800 kb disk and a CBM 1581 disk is.
 *   (check: http://vanbc.wimsey.com/~danf/software
 *           http://www.npsnet.com/danf/software)
 *
 *
 * Basic implementations by Ciriaco Garca de Celis <ciri@gui.uva.es>
 *   His util 765Debug, Version 5.0 is great for learning dma based
 *   direct floppy disk controller programming.
 *   (check: ftp://ftp.vse.cz/pub/misc/2m/current/765d50sr.zip)
 *
 * Check out for his floppy disk utils 2M and 2MGUI, the last words
 * in improving floppy disk storage capacity.
 *   ftp://ftp.vse.cz/pub/misc/2m/current
 *
 *
 * For additional informations to FDC programming check:
 *     http://developer.intel.com/design/periphrl/datashts/290468.htm
 *     http://developer.intel.com/design/periphrl/datashts/290474.htm
 *   and get the intel 82078 CHMOS Single-Chip Floppy Disk Controller
 *   PDF document:
 *     ftp://download.intel.com/design/periphrl/datashts/29046803.pdf
 *     ftp://download.intel.co.jp/design/periphrl/datashts/29046803.pdf
 *     ftp://download.intel.com/design/periphrl/datashts/29047403.pdf
 *     ftp://download.intel.co.jp/design/periphrl/datashts/29047403.pdf
 *
 *
 *   National Semiconductor has also some pages about their PC
 *   compatible controllers:
 *     http://www.national.com/pf/DP/DP8473.html
 *     http://www.national.com/pf/PC/PC8477B.html
 *
 *
 * Another good source for floppy disk controller programming information
 * are the linux kernal sources, you could have a look into:
 *     http://src.openresources.com/linux-2.2.1/S/include%20linux%20fdreg.h.html
 *     http://src.openresources.com/linux-2.2.1/S/drivers%20block%20floppy.c.html
 * especially the parameter table explanation
 *     http://src.openresources.com/linux-2.2.1/S/drivers%20block%20floppy.c.html#332
 *
 *
 * Now, that I started talking about Linux: Check out the most massive
 * attack against floppy disks ever made, Alain Knaff's fdutils:
 *     http://alain.knaff.linux.lu/fdutils/index.html
 *     http://fdutils.linux.lu/index.html
 */

#ifndef __Transfer_Operations__
#define __Transfer_Operations__

#include <stdlib.h>
#include <stdio.h>
#include "1581hlvl.h"
#include "frmheadr.h"

// extern unsigned char	 CylinderBuffer[2*TrBufSize];
extern unsigned char	*CylinderBuffer;
extern unsigned char	*TransferBuffer;

extern FILE				*FileDescriptor;

extern unsigned int	 FloppyDriveNo;
extern unsigned int	 MultipleDisks;
extern unsigned int	 SingleSectorC;
	// the following three would have been better combined into one "xferM" variable
extern FDprmSet		 FormattingDsk;
extern xferM			 WritingToDisk;
extern xferM			 VerifyWriting;

extern unsigned int	 FrmtNWrt82078;
extern unsigned int	 NumberRetries;
extern unsigned int	 SectorIntrlve;
extern unsigned char	 BAMcopymarker[21];	// BAM copy track TransferBuffer (168 tracks)

typedef enum{
	RetSUCC=  0,	// operation completed successfully, or the multiple disks copy
						// loop has been exited by user request
   RetHELP=  1,	// command line parameter error or help called
	RetABRT=  2,	// operation has been aborted by user request

	RetCNFG= 10,	// configuration error, wrong drive type (no 3,5" drive)
	RetWPRT= 11,	// disk write protect error (format protect error, too)
	RetXFER= 12,	// unrecoverable disk transfer error on at least 1 track
						// (read, write, format, or verify errors)
	RetUNKW= 13,	// no or CBM incompatible disk found with switch /N

	RetOPEN= 20,	// image file open error
	RetREAD= 21,	// image file read error
	RetWRTE= 22,	// image file write error
	RetCPTL= 23,	// no CBM compatible image file(s) found

	RetINDX= 30,	// automatic filename (index) generation error
	RetEXNX= 31,	// automatic extension generation error

	RetTBAE= 40,	// track buffer allocation error

	RetFMFX=100		//	1xx -  special debug return codes
	}ExitCode;

void	exit1581Copy		(ExitCode retcode, unsigned int stdmsg);
ENo	transferDiskTrack	(xferM xfer ,uint8 cyl, uint8 side);
int	getTrackAllocation(FDprmSet type);
int	writeDefaultDisk	(void);

char	*buildInxName		(char *inStr);
char	*changeExtension	(char *inStr, int force, FDprmSet Set);

#endif
